"""A collection of dict tools.
"""
from collections import OrderedDict
from copy import copy
import operator
import re

from utensils.stringutils import normalize

ARRAY_ACCESSOR = re.compile(r'(.*)\[(.*?)\]')

def flatten(dict_, key='children', children=[]):
    """
    Flattens nested dictionaries that have a nesting structure based on key.

    @param dict_: dict, starting with the parent node
    @param key: str, key on which to look for the children
    @return: list, list of children
    """
    flat = []
    if key in dict_:
        for child in dict_[key]:
            flat.extend(flatten(child, key))
        del(dict_[key])
    flat.append(dict_)
    return flat

def inverse(dict_):
    """
    @param dict_: dict
    @return dict_
    """
    return dict((v,k) for k, v in dict_.iteritems())

def order(dict_, by_value=False, reverse=False):
    """
    @param dict_: dict
    @param by_value: bool
    @return: dict_
    """
    def _getter(t):
        return t[1] if by_value else t[0]
    return OrderedDict(sorted(dict_.items(),
        key=_getter,
        reverse=reverse,
        ))

def transform(dict_, mapper, clone=False):
    """
    Transform the dict with the path mapping rules passed in the mapper. Does
    not modify the passed in dict_

    @param dict_, dict
    @param mapper, dict(key, value)
    @param clone: bool, clone initial dict and start writing on it as opposed
    to starting with an empty dict.
    @return dict_
    """

    data = copy(dict_) if clone else {}
    for from_path, to_path in mapper.items():
        set_dotted(data, to_path, get_dotted(dict_, from_path))
    return data

def get_dotted(data, field, default=None, delimiter='.',
        do_normalize=False,
        ):
    """
    Get a nested subfield inside data using Mongo style dotted notation
    notation. For example:
        data = {'a': 1, 'b': {'c': 2, 'd': {'e': 3}}}
        get_dotted_field(data, 'b.d.e') => 3

    Also allows array accessors:
        foo[@bla=3].bar: Choose the element in foo, whose attribute bla equals 3
        foo[3].bar: Choose index 3 in the foo array.

    Note that it is type insensitive, i.e. in comparisions '3' and 3 will
    evaluate to the same.
    """
    components = field.split(delimiter)
    components.reverse()
    while components and isinstance(data, dict):
        component = components.pop()
        accessor = None
        match = re.match(ARRAY_ACCESSOR, component)
        if match:
            component = match.group(1)
            accessor = match.group(2)
        data = data.get(component)
        if accessor:
            # Dict accessor
            if "@" in accessor:
                params = {}
                for pairs in accessor.split('@')[1:]:
                    params[pairs.split('=')[0]] = pairs.split('=')[1]
                # Kind of hacky, but let's handle the case where this should
                # be a list, but turns out a dict, such as in Gap's case.
                data = data if isinstance(data, list) else [data]
                for item in data:
                    for key, value in params.items():
                        if do_normalize:
                            val1 = normalize(str(item[key]))
                            val2 = normalize(str(value))
                        else:
                            val1 = str(item[key])
                            val2 = str(value)
                        if val1 != val2:
                            break
                    # If we iterated over all the params, and did not break
                    # then we are good.
                    else:
                        data=item
                        break

            # List accessor
            elif "*" in accessor:
                cur_fields = delimiter.join(components[::-1])
                return [get_dotted(d, cur_fields) for d in data]
            else:
                data = operator.getitem(data, int(accessor))

    # If there are components left, the final dest wasn't reached.
    if components or data == None:
        return default
    return data

def set_dotted(data, field, value, delimiter='.'):
    """Set a nested subfield inside data using Mongo style dotted notation
    notation. For example:
        data = {'a': 1, 'b': {'c': 2, 'd': {'e': 3}}}
        set_dotted_field(data, 'b.d.e', 4)
        data == {'a': 1, 'b': {'c': 2, 'd': {'e': 4}}}
    """
    if not delimiter in field:
        data[field] = value
    else:
        next_field, remaining_fields = field.split(delimiter, 1)

        # If next_field isn't there, create it so we can move on.
        if next_field not in data:
            data[next_field] = {}

        set_dotted(data[next_field], remaining_fields, value, delimiter=delimiter)

def unicode_keys(data):
    return apply_to_dict(data,
                         lambda x: unicode(x) if isinstance(x,str) else x)

def apply_to_dict(data, _function):
    if type(data) == dict:
        data = dict(data) # copy it first
        for key in data:
            data[key] = apply_to_dict(data[key], _function)
    elif type(data) == list:
        data = list(data) # copy it first
        for i, item in enumerate(data):
            data[i] = apply_to_dict(item, _function)
    else:
        data = _function(data)
    return data

def replace_unicode_keys(data, recurse=False):
    """ Convert top-level unicode strings to regular strings so the
    dict can be passed as keyword args

    @param data: Dict to replace unicode keys of.
    @param recurse: Recursively replace sub-dicts. Default: False
    """
    # Copy dict so original isn't modified.
    if not recurse:
        return dict([(str(key), data[key]) for key in data])
    new_data = {}
    for key in data:
        if type(data[key]) == dict:
            new_data[str(key)] = replace_unicode_keys(data[key], recurse=True)
        else:
            new_data[str(key)] = data[key]
    return new_data

def dict_without_keys(src_dict, keys_to_exclude):
    """Copies a dict, leaving out some keys if there.

    @param src_dict: Dict to copy.
    @param keys_to_exclude: A list of keys to exclude. All these don't have to
        be in the src_dict. They'll be ignored if they're not.
    @return: A copy of src_dict without keys_to_exclude.
    """
    return dict( (key, value) for key, value in src_dict.iteritems() if key not in keys_to_exclude )

def get_from_list(items, key, value, default=None):
    """Get an item from a list of collections based on a key/value.

    @param items: A list of collections.
    @param key: The key to look up in the items in the list.
    @return: The first item that has the given key with the given value, or the default.
    """
    for item in items:
        try:
            if item[key] == value:
                return item
        except KeyError:
            pass
    return default

def to_dict_by_key(col, key_key, value_key=None, include_key=False):
    """
    Turn a list of collections into a dict by a key in the items, optionally
    only using a value.
    Ex:
      col = [{'k': 'a', 'n': 1}, {'k': 'b', 'n': 2}, {'n': 3}]
      to_dict_by_key(col, 'k') --> {'a': {'n': 1}, 'b': {'n': 2}}
      to_dict_by_key(col, 'k', include_key=True) --> {'a': {'k': 'a', 'n': 1}, 'b': {'k': 'b', 'n': 2}}
      to_dict_by_key(col, 'k', 'v') --> {'a': 1, 'b': 2}

    Items that don't contain key are ignored.

    @param col: collection, Collection to turn into a dict.
    @param key_key: str, Key to look for in items and turn into keys in resulting dict.
    @param value_key: Key to look for in items and turn into values in resulting dict.
    @return: A dict keyed by things in the collection.
    """
    if value_key != None:
        return dict( (item[key_key], item[value_key]) for item in col if key_key in item )
    elif not include_key:
        return dict( (item[key_key], dict_without_keys(item, [key_key])) for item in col if key_key in item )
    else:
        return dict( (item[key_key], item) for item in col if key_key in item )

def combine_dicts(dicts, fn=lambda values: sum(values), default=None):
    """Combine values of multiple dicts into one using a function.
    If no default is given, keys that are missing from one or more dicts are
    ignored.

    @param dicts: A collection of dicts to combine.
    @param fn: The function to run over multiple values from the dicts for each
        key, before putting into combined dict.
    @param default: For dicts that don't have a particular key, use this value
        before passing into fn for combining. If default == None, ignore the
        entire key (don't include in combined dict) if any input dicts don't
        have the key.
    @return: A dict containing the combination of the input dicts.
    """
    # Copy dicts in case it's a generator.
    dicts = list(dicts)
    if default == None:
        all_keys = reduce(lambda a,b: set(a).intersection(b), (d.keys() for d in dicts))
        return dict( (key, fn([d[key] for d in dicts])) for key in all_keys )
    else:
        all_keys = reduce(lambda a,b: set(a).union(b), (d.keys() for d in dicts))
        return dict( (key, fn([d.get(key, default) for d in dicts])) for key in all_keys )

def deep_items(d, delimeter='.'):
    """
    Travsere the dict (which could be a dict of dicts) and return a list of
    path, value pairs.

    @param d: dict
    @param delimterer: str, optional.
    @return: list(str, Object)
    """
    pairs = []
    for k, v in d.items():
        if isinstance(v, dict):
            cur_pairs = deep_items(v, delimeter)
            pairs.extend([(k + delimeter + cur_k, cur_v) for (cur_k, cur_v) in cur_pairs])
        else:
            pairs.append((k, v))
    return pairs

def get_value_for_key(data, key):
    """
    Recursively iterate over a dict to find the value for a key

    @param data: dict
    @param key: str
    @return: Object
    """
    if isinstance(data, dict):
        for cur_key, val in data.items():
            if cur_key == key:
                return val
            to_return = get_value_for_key(val, key)
            if to_return:
                return to_return
    elif isinstance(data, list):
        for item in data:
            to_return = get_value_for_key(item, key)
            if to_return:
                return to_return
    return None



# This Python file uses the following encoding: utf-8
import logging
import HTMLParser
import posixpath
import random
import re
import string

PHONE_NUMBER_PATTERN = re.compile(r'(\d{3})[-).(] *(\d{3})[-.](\d{4})')

TURKISH_TO_LATIN_CHAR_MAP = {
        u'ç':u'c',
        u'ı':u'i',
        u'ğ':u'g',
        u'ö':u'o',
        u'ü':u'u',
        u'ş':u's',
        u'â':u'a',
        }

def generate_str(size=32, chars=string.ascii_uppercase + string.digits):
    return ''.join(random.choice(chars) for x in range(size))

def string_contains(string, keywords, lower=True, full_word=False):
    """
    @param word: str
    @param keywords: list(str), check if any one of the keywords matches
    @param lower: bool
    @param full_word: bool, only return true if string contains a full_word
    that matches one of the keywords
    @return: bool
    """
    if not string:
        return False
    if lower:
        string = string.lower()
        words = [w.lower() for w in keywords]
    if not full_word:
        return bool([w for w in words if w in string])
    for keyword in keywords:
        for word in string.split():
            if word == keyword:
                return True
    return False

def _callback(matches):
    id = matches.group(1)
    try:
        return unichr(int(id))
    except:
        return id

def decode_unicode_references(data):
    return re.sub("&#(\d+)(;|(?=\s))", _callback, data)

def basename(path, ext=True):
    """
    @param path: str
    @param ext: bool, whether or not to return the extension
    @return: bool
    """
    name = posixpath.basename(path)
    if not ext:
        name = posixpath.splitext(name)[0]
    return name

def is_upper(s):
    """
    @param s: str
    @return: bool
    """
    if not s:
        return False
    return all(map(lambda c: c.isalpha() and c.istitle(), s))

def parse_int_or_string(val):
    """
    @param val: str
    @return: int|str|None
    """
    if not val:
        return val
    try:
        return int(val)
    except:
        return val

def parse_price(price, fun=max):
    """
    @param price: str
    @param fun: max | min : determine max or min of price range when there is conflicts
    @return: float
    taking
    """
    price = price.lower().replace('for', '|')
    price = re.sub('[\(\)a-z\s$&;:]', '', price.lower())
    # Strip leading / trailing .'s
    price = price.strip('.')
    # Combine comma-separation used for denoting 000s
    if '|' in price:
        quantity, amount = price.split('|')
        return float(amount) / float(quantity)
    return float(fun(price.replace(',', '').split('-'))) #'$308.00 - $440.00'

def normalize(s):
    """
    @param str: s
    @return: str
    """
    if not s:
        return s
    s = s.replace('&amp;', '&')
    s = s.replace('&nbsp;', ' ')
    s = s.strip().lower()
    return s

def unformat_phone_number(phone):
    """
    input: 516.873.7380 output: 5168737380
    input: 516-873-7380 output: 5168737380
    input: (212) 431-2686 output: 2124312686
    """
    phone = phone.replace(' ', '')
    h = HTMLParser.HTMLParser()
    phone = h.unescape(phone)
    match = PHONE_NUMBER_PATTERN.search(phone)
    if match:
        return ''.join(match.groups())
    phone = re.sub(r'[^\d]+', '', phone)

    if phone and phone[0] == '1':
        phone = phone[1:]

    if len(phone) != 10:
        raise ValueError("phone number %s is not 10 digits long, potential error" % phone)
    return phone

def to_upper_first_chars(str_):
    return ' '.join([word.capitalize() for word in str_.lower().split(' ')])

def to_latin(str_):
    for t_char, l_char in TURKISH_TO_LATIN_CHAR_MAP.items():
        str_ = str_.replace(t_char, l_char)
    return str_

def to_float(str_):
    """
    @param str_: str
    @return: float
    """
    return float(str_) if str_ else None

def to_array(str_, delimeter=','):
    """
    @param str_: str
    @param delimeter: str
    @return: list
    """
    if not str_:
        return []
    return str_.split(delimeter)

def to_bool(str_):
    """
    Convert various input for specifying a boolean to a Bool.

    @param str_: str, input to parse
    @return: bool
    """

    if str_ in [True, 1, 'True', 'true', '1']:
        return True

    if str_ in [None, False, 0, 'False', 'false', '0']:
        return False

    # As a last resort try interpreting as a number, letting it
    # throw an exception if it doesn't work.
    try:
        return int(str_) != 0
    except:
        return False

def _suffix(d):
    return 'th' if 11<=d<=13 else {1:'st',2:'nd',3:'rd'}.get(d%10, 'th')

def custom_strftime(format, t):
    return t.strftime(format).replace('{S}', str(t.day) + _suffix(t.day))

def slugify(n):
    """
    TODO: Normally do not use this, and use django's version. We are using
    it here for backwards compatibality.

    @param n: str
    @return: str
    """
    n = re.sub(r'[^a-zA-Z0-9 ]', '', n)
    n = re.sub(r'\s+', '-', n).lower()
    return n



from distutils.core import setup
import utensils

requires = []

packages = [
    'utensils',
]

setup(
    name='utensils',
    author='Eytan Daniyalzade',
    author_email='eytan85@gmail.com',
    url='http://daniyalzade.com',
    packages=packages,
    description='Library of utilities for python',
    long_description=open('README.rst').read(),
    version=utensils.__version__,
    data_files=[
        ('', ['README.rst']),
    #    ('utensils', [
    #        'utensils/base.html',
    #        'utensils/style.css',
    #        ]),
        ],
    package_dir={
        'utensils': 'utensils'
        },
    #package_data={
    #    'utensils': [
    #        '*.html',
    #        '*.css',
    #        ],
    #    },
    install_requires=requires,
    include_package_data=True,
)
